#define NODEFLEX 0

#define NF_PRECISION highp

#ifdef ENABLE_LIGHTING
#undef ENABLE_LIGHTING
#endif
#ifdef ENABLE_DIFFUSE_LIGHTING
#undef ENABLE_DIFFUSE_LIGHTING
#endif
#ifdef ENABLE_SPECULAR_LIGHTING
#undef ENABLE_SPECULAR_LIGHTING
#endif
#ifdef ENABLE_TONE_MAPPING
#undef ENABLE_TONE_MAPPING
#endif

#define SC_USE_USER_DEFINED_VERTEX_PROCESSING
#define SC_USE_USER_DEFINED_VS_MAIN
#define SC_SKIP_DUPLICATE_INCLUDES
#define SC_REMOVE_ENVMAP_FROM_AMBIENT_LIGHT_STRUCT

#include <std.glsl>
#include <std_vs.glsl>
#include <std_fs.glsl>
#pragma once

#ifndef GLES20STD_TEXTURE
#define GLES20STD_TEXTURE

highp float depthToGlobal(highp float depth,highp vec2 projectionMatrixTerms){
	highp float m22=projectionMatrixTerms.x;
	highp float m32=projectionMatrixTerms.y;
	depth=depth*2.0 -1.0;
	return m32/(-depth -m22);
}

highp float depthToLocal(highp float depth,highp vec2 projectionMatrixTerms){
	highp float m22=projectionMatrixTerms.x;
	highp float m32=projectionMatrixTerms.y;
	depth=depth !=0.0 ? (-m22 -m32/depth) : 0.0;
	return depth*0.5+0.5;
}

#ifdef GL_ES
#if __VERSION__ >=300
#define SC_SUPPORT_2D_ARRAY_TEXTURE 1
#define SC_SUPPORT_3D_TEXTURE 1
#define texture2DArray texture
#define texture3D texture
#define textureCube texture
#else
#define SC_SUPPORT_2D_ARRAY_TEXTURE 0
#define SC_SUPPORT_3D_TEXTURE 0
#endif
#else
#define SC_SUPPORT_2D_ARRAY_TEXTURE 1
#define SC_SUPPORT_3D_TEXTURE 1
#endif

#define CLAMP_UV(uv,uvMin,uvMax,useClampToBorder,borderClampFactor){\
	float clampedUv=clamp(uv,uvMin,uvMax);\
\
	if (useClampToBorder){\
		borderClampFactor*=step(abs(uv -clampedUv),0.00001);\
	}\
\
	uv=clampedUv;\
}

#define SOFTWARE_WRAP(uv,softwareWrapMode,useClampToBorder,clampToBorderFactor){\
	if (softwareWrapMode==SOFTWARE_WRAP_MODE_CLAMP_TO_EDGE || softwareWrapMode==SOFTWARE_WRAP_MODE_CLAMP_TO_BORDER){\
		CLAMP_UV(uv,0.0,1.0,useClampToBorder,clampToBorderFactor);\
	} else if (softwareWrapMode==SOFTWARE_WRAP_MODE_REPEAT){\
		uv=fract(uv);\
	} else if (softwareWrapMode==SOFTWARE_WRAP_MODE_MIRRORED_REPEAT){\
		float uvFract=fract(uv);\
		float uvInt=uv -uvFract;\
		float uvOdd=step(0.25,fract(uvInt*0.5));\
		uv=mix(uvFract,1.0 -uvFract,clamp(uvOdd,0.0,1.0));\
	}\
}

#ifdef GL_ES
#define QUANTIZE_UV(A) A
#else
#define QUANTIZE_UV(A) A - (fract(A*4096.0)/4096.0);
#endif

#ifdef FRAGMENT_SHADER
#define TEXTURE2D(sampler,uv,mipBias) texture2D(sampler,uv,mipBias)
#define TEXTURE2DARRAY(sampler,uv,mipBias) texture2DArray(sampler,uv,mipBias)
#define TEXTURE3D(sampler,uv,mipBias) texture3D(sampler,uv,mipBias)
#define TEXTURECUBE(sampler,uv,mipBias) textureCube(sampler,uv,mipBias)
#else
#define TEXTURE2D(sampler,uv,mipBias) texture2D(sampler,uv)
#define TEXTURE2DARRAY(sampler,uv,mipBias) texture2DArray(sampler,uv)
#define TEXTURE3D(sampler,uv,mipBias) texture3D(sampler,uv)
#define TEXTURECUBE(sampler,uv,mipBias) textureCube(sampler,uv)
#endif

#define SAMPLE_TEXTURE(tex,sampler,uvIn,useUvTransform,uvTransform,softwareWrapModeU,softwareWrapModeV,useUvMinMax,uvMinMax,useClampToBorder,borderColor,mipBias){\
	vec2 uvTemp=uvIn;\
\
	float clampToBorderFactor=1.0;\
\
	SOFTWARE_WRAP(uvTemp.x,softwareWrapModeU,useClampToBorder && !useUvMinMax,clampToBorderFactor);\
	SOFTWARE_WRAP(uvTemp.y,softwareWrapModeV,useClampToBorder && !useUvMinMax,clampToBorderFactor);\
\
	if (useUvMinMax){\
		CLAMP_UV(uvTemp.x,uvMinMax.x,uvMinMax.z,useClampToBorder,clampToBorderFactor);\
		CLAMP_UV(uvTemp.y,uvMinMax.y,uvMinMax.w,useClampToBorder,clampToBorderFactor);\
	}\
\
	if (useUvTransform){\
		uvTemp=vec2(uvTransform*vec3(uvTemp,1.0));\
	}\
\
	uvTemp=QUANTIZE_UV(uvTemp);\
\
	tex=TEXTURE2D(sampler,uvTemp,mipBias);\
\
	if (useClampToBorder){\
		tex=mix(borderColor,tex,clampToBorderFactor);\
	}\
}

#if SC_SUPPORT_2D_ARRAY_TEXTURE
#define SAMPLE_TEXTURE_2D_ARRAY(tex,sampler,uvwIn,useUvTransform,uvTransform,mipBias){\
	vec2 uvTemp=uvwIn.xy;\
\
	if (useUvTransform){\
		uvTemp=vec2(uvTransform*vec3(uvTemp,1.0));\
	}\
\
	uvTemp=QUANTIZE_UV(uvTemp);\
\
	tex=TEXTURE2DARRAY(sampler,vec3(uvTemp,uvwIn.z),mipBias);\
}
#else
#define SAMPLE_TEXTURE_2D_ARRAY(tex,sampler,uvwIn,useUvTransform,uvTransform,mipBias){\
	tex=vec4(0.0,0.0,0.0,1.0);\
}
#endif

#if SC_SUPPORT_3D_TEXTURE
#define SAMPLE_TEXTURE_3D(tex,sampler,uvwIn,mipBias){\
	vec3 uvwTemp=uvwIn;\
\
	uvwTemp=QUANTIZE_UV(uvwTemp);\
\
	tex=TEXTURE3D(sampler,uvwTemp,mipBias);\
}
#else
#define SAMPLE_TEXTURE_3D(tex,sampler,uvwIn,mipBias){\
	tex=vec4(0.0,0.0,0.0,1.0);\
}
#endif

#define SAMPLE_TEXTURE_CUBE(tex,sampler,uvwIn,mipBias){\
	vec3 uvwTemp=uvwIn;\
\
	uvwTemp=QUANTIZE_UV(uvwTemp);\
\
	tex=TEXTURECUBE(sampler,uvwTemp,mipBias);\
}

#define SAMPLE_DEPTH_TEXTURE(tex,sampler,uvIn,useUvTransform,uvTransform){\
	vec2 uvTemp=uvIn;\
\
	if (useUvTransform){\
		uvTemp=vec2(uvTransform*vec3(uvTemp,1.0));\
	}\
\
	uvTemp=QUANTIZE_UV(uvTemp);\
\
	tex=texture2D(sampler,uvTemp).r;\
}

#define SAMPLE_DEPTH_TEXTURE_NDC(tex,sampler,uvIn,useUvTransform,uvTransform,projectionMatrixTerms){\
	vec2 uvTemp=uvIn;\
\
	if (useUvTransform){\
		uvTemp=vec2(uvTransform*vec3(uvTemp,1.0));\
	}\
\
	uvTemp=QUANTIZE_UV(uvTemp);\
\
	float depth=texture2D(sampler,uvTemp).r;\
\
	tex=depthToGlobal(depth,projectionMatrixTerms);\
}

#ifdef CUSTOM_PREPROCESSOR

#endif

#ifdef SC_TEXTURE_EXTERNAL
#define sc_SamplerExternalOES samplerExternalOES
#else
#define sc_SamplerExternalOES sampler2D
#endif

#ifdef FRAGMENT_SHADER

#if defined(GL_ES) && (__VERSION__ < 300) && !defined(GL_OES_standard_derivatives)
#define dFdx(A) (A)
#define dFdy(A) (A)
#define fwidth(A) (A)
#endif

#ifdef SC_TEXTURE_EXTERNAL
#define sc_textureExternalOESLod(tex,size,uv,lod) texture2D(tex,uv)

#else
vec4 sc_texture2DLod(sampler2D tex,vec2 textureSize,vec2 uv,float lod){
#if (__VERSION__==120)
#define SC_TESTURE2DLOD_IS_SUPPORTED
	return texture2DLod(tex,uv,lod);
#elif (__VERSION__ >=300)
#define SC_TESTURE2DLOD_IS_SUPPORTED
	return textureLod(tex,uv,lod);
#elif defined(GL_EXT_shader_texture_lod)
#define SC_TESTURE2DLOD_IS_SUPPORTED
	return texture2DLodEXT(tex,uv,lod);
#elif defined(GL_OES_standard_derivatives)
#define SC_TESTURE2DLOD_IS_SUPPORTED
	vec2 texels=uv*textureSize;
	float dudx=dFdx(texels.x);
	float dvdx=dFdx(texels.y);
	float dudy=dFdy(texels.x);
	float dvdy=dFdy(texels.y);
	float rho=max(length(vec2(dudx,dvdx)),length(vec2(dudy,dvdy)));
	float mu=max(abs(dudx),abs(dudy));
	float mv=max(abs(dvdx),abs(dvdy));
	float rho2=max(mu,mv);
	float mip=log2(rho2);
	float bias=lod -mip;
	return texture2D(tex,uv,bias);
#else
	return texture2D(tex,uv,lod);
#endif
}

#define sc_textureExternalOESLod(tex,size,uv,lod) sc_texture2DLod(tex,size,uv,lod)
#endif

#if SC_SUPPORT_2D_ARRAY_TEXTURE
vec4 sc_texture2DArrayLod(sc_FragmentPrecision sampler2DArray tex,vec3 uv,float lod){
#if (__VERSION__==120)
	return texture2DArrayLod(tex,uv,lod);
#elif (__VERSION__ >=300)
	return textureLod(tex,uv,lod);
#elif defined(GL_EXT_shader_texture_lod)
	return texture2DArrayLodEXT(tex,uv,lod);
#else
	return texture2DArray(tex,uv,lod);
#endif
}
#else

#define sc_texture2DArrayLod(tex,uv,lod) vec4(0,0,0,1)

#endif

#endif

#if SC_SUPPORT_3D_TEXTURE
vec4 sc_texture3DLod(sc_FragmentPrecision sampler3D tex,vec3 uv,float lod){
#if (__VERSION__==120)
	return texture3DLod(tex,uv,lod);
#elif (__VERSION__ >=300)
	return textureLod(tex,uv,lod);
#elif defined(GL_EXT_shader_texture_lod)
	return texture3DLodEXT(tex,uv,lod);
#else
	return texture3D(tex,uv,lod);
#endif
}
#else

#define sc_texture3DLod(tex,uv,lod) vec4(0,0,0,1)

#endif

vec4 sc_textureCubeLod(samplerCube tex,vec3 uv,float lod){
#if (__VERSION__ < 120)
	return textureCube(tex,uv);
#elif (__VERSION__==120)
	return textureCubeLod(tex,uv,lod);
#elif (__VERSION__ >=300)
	return textureLod(tex,uv,lod);
#elif defined(GL_EXT_shader_texture_lod)
	return textureCubeLodEXT(tex,uv,lod);
#else
	return textureCube(tex,uv,lod);
#endif
}

#endif

#define SCENARIUM

#ifdef GL_ES
#define MOBILE
#endif

#if SC_DEVICE_CLASS >=SC_DEVICE_CLASS_C && (!defined(MOBILE) || defined(GL_FRAGMENT_PRECISION_HIGH))
#define DEVICE_IS_FAST
#endif

#if defined(ENABLE_LIGHTING) || defined(ENABLE_EMISSIVE) || defined(ENABLE_VERTEX_COLOR_EMISSIVE) || defined(ENABLE_SIMPLE_REFLECTION) || defined(ENABLE_RIM_HIGHLIGHT) || defined(ENABLE_TONE_MAPPING)
#define SC_ENABLE_SRGB_EMULATION_IN_SHADER
#endif

varying vec4 varColor;

#ifndef UTILS_GLSL
#define UTILS_GLSL

#ifndef PI
#define PI 3.141592653589793238462643383279
#endif

#define SC_EPSILON 1e-6

#ifndef MAYA

float saturate(float value){
	return clamp(value,0.0,1.0);
}

vec2 saturate(vec2 value){
	return clamp(value,0.0,1.0);
}

vec3 saturate(vec3 value){
	return clamp(value,0.0,1.0);
}

#endif

#if defined(MAYA) || defined(SUBSTANCE)

float srgbToLinear(float x){
	return x <=0.04045 ? x*0.0773993808 : pow((x+0.055)/1.055,2.4);
}

float linearToSrgb(float x){
	return x <=0.0031308 ? 12.92*x : 1.055*pow(x,0.41666) -0.055;
}

#else

float srgbToLinear(float x){
#ifdef SC_ENABLE_SRGB_EMULATION_IN_SHADER
#ifdef DEVICE_IS_FAST
	return pow(x,2.2);
#else
	return x*x;
#endif
#else
	return x;
#endif
}

float linearToSrgb(float x){
#ifdef SC_ENABLE_SRGB_EMULATION_IN_SHADER
#ifdef DEVICE_IS_FAST
	return pow(x,1.0/2.2);
#else
	return sqrt(x);
#endif
#else
	return x;
#endif
}

#endif

vec3 srgbToLinear(vec3 color){
	return vec3(srgbToLinear(color.r),srgbToLinear(color.g),srgbToLinear(color.b));
}

vec3 linearToSrgb(vec3 color){
	return vec3(linearToSrgb(color.r),linearToSrgb(color.g),linearToSrgb(color.b));
}

#endif

#ifndef BLEND_MODES_GLSL
#define BLEND_MODES_GLSL

#include <std_fs.glsl>

#ifndef BLEND_MODES_EYECOLOR_GLSL
#define BLEND_MODES_EYECOLOR_GLSL

#ifndef RGBHSL_GLSL
#define RGBHSL_GLSL

vec3 RGBtoHCV(vec3 rgb)
{
	vec4 p=(rgb.g < rgb.b) ? vec4(rgb.bg,-1.0,2.0/3.0) : vec4(rgb.gb,0.0,-1.0/3.0);
	vec4 q=(rgb.r < p.x) ? vec4(p.xyw,rgb.r) : vec4(rgb.r,p.yzx);

	float c=q.x -min(q.w,q.y);
	float h=abs((q.w -q.y)/(6.0*c+1e-7)+q.z);
	float v=q.x;

	return vec3(h,c,v);
}

vec3 RGBToHSL(vec3 rgb)
{
	vec3 hcv=RGBtoHCV(rgb);

	float lum=hcv.z -hcv.y*0.5;
	float sat=hcv.y/(1.0 -abs(2.0*lum -1.0)+1e-7);

	return vec3(hcv.x,sat,lum);
}

vec3 HUEtoRGB(float hue)
{
	float r=abs(6.0*hue -3.0) -1.0;
	float g=2.0 -abs(6.0*hue -2.0);
	float b=2.0 -abs(6.0*hue -4.0);
	return clamp(vec3(r,g,b),0.0,1.0);
}

vec3 HSLToRGB(vec3 hsl)
{
	vec3 rgb=HUEtoRGB(hsl.x);
	float c=(1.0 -abs(2.0*hsl.z -1.0))*hsl.y;
	rgb=(rgb -0.5)*c+hsl.z;
	return rgb;
}

#endif

#ifdef BLEND_MODE_REALISTIC
#define COLOR_MODE 0
#endif

#ifdef BLEND_MODE_DIVISION
#define COLOR_MODE 1
#endif

#ifdef BLEND_MODE_BRIGHT
#define COLOR_MODE 2
#endif

#ifdef BLEND_MODE_FORGRAY
#define COLOR_MODE 3
#endif

#ifdef BLEND_MODE_NOTBRIGHT
#define COLOR_MODE 4
#endif

#ifdef BLEND_MODE_INTENSE
#define COLOR_MODE 5
#endif

#ifdef COLOR_MODE

uniform float	correctedIntensity;
uniform sampler2D intensityTexture;

#if COLOR_MODE==0 || COLOR_MODE==3 || COLOR_MODE==4

float transformSingleColor(float original,float intMap,float target){
	return original/pow((1.0 -target),intMap);
}

#endif
#if COLOR_MODE==1

float transformSingleColor(float original,float intMap,float target){
	return original/(1.0 -target);
}

#endif
#if COLOR_MODE==2

float transformSingleColor(float original,float intMap,float target){
	return original/pow((1.0 -target),2.0 -2.0*original);
}

#endif

#if COLOR_MODE !=5

vec3 transformColor(float yValue,vec3 original,vec3 target,float weight,float intMap){
	vec3 tmpColor;
	tmpColor.r=transformSingleColor(yValue,intMap,target.r);
	tmpColor.g=transformSingleColor(yValue,intMap,target.g);
	tmpColor.b=transformSingleColor(yValue,intMap,target.b);
	tmpColor=clamp(tmpColor,0.0,1.0);
	vec3 resColor=mix(original,tmpColor,weight);
	return resColor;
}

#endif

#if COLOR_MODE==5

vec3 transformColor(float yValue,vec3 original,vec3 target,float weight,float intMap){
	vec3 hslOrig=RGBToHSL(original);
	vec3 res;
	res.r=target.r;
	res.g=target.g;
	res.b=hslOrig.b;
	res=HSLToRGB(res);
	vec3 resColor=mix(original,res,weight);
	return resColor;
}

#endif

float unpack1(float inp,float mul){
	return inp*mul;
}

float unpack2(vec2 inp,float mul){
	return (inp[0]*256.0+inp[1])/257.0*mul;
}

float unpack3(vec3 inp,float mul){

	return (inp[0]*256.0+inp[1]+inp[2]/256.0)/(256.0+1.0+1.0/256.0)*mul;
}

float getYValue(vec3 rgb){
	return 0.299*rgb.r+0.587*rgb.g+0.114*rgb.b;
}

vec3 eyeColorBlend(vec3 texColor,vec3 resColor)
{
	float newYValue=getYValue(texColor);

	float weight=1.0;
	float fragmentCorrectedIntensity=pow(newYValue,1.0/correctedIntensity);
	vec3 intenseMapCompressed=texture2D(intensityTexture,vec2(fragmentCorrectedIntensity,0.5)).rgb;
	float intenseMapValue=unpack3(intenseMapCompressed,16.0);

#if COLOR_MODE==3
	intenseMapValue=max(intenseMapValue,1.0);
#endif
#if COLOR_MODE==4
	intenseMapValue=min(intenseMapValue,1.0);
#endif

	vec3 newColor=transformColor(newYValue,texColor,resColor,weight,intenseMapValue);
	return newColor;
}

#define definedBlend eyeColorBlend

#endif

#endif

#ifdef FRAGMENT_SHADER
#ifdef sc_BlendMode_Custom

vec3 ContrastSaturationBrightness(vec3 color,float brt,float sat,float con)
{

	const float AvgLumR=0.5;
	const float AvgLumG=0.5;
	const float AvgLumB=0.5;

	const vec3 LumCoeff=vec3(0.2125,0.7154,0.0721);

	vec3 AvgLumin=vec3(AvgLumR,AvgLumG,AvgLumB);
	vec3 brtColor=color*brt;
	vec3 intensity=vec3(dot(brtColor,LumCoeff));
	vec3 satColor=mix(intensity,brtColor,sat);
	vec3 conColor=mix(AvgLumin,satColor,con);
	return conColor;
}

float BlendAddf(float base,float blend){
	return min(base+blend,1.0);
}
float BlendSubtractf(float base,float blend){
	return max(base+blend -1.0,0.0);
}
float BlendLinearDodgef(float base,float blend){
	return min(base+blend,1.0);
}
float BlendLinearBurnf(float base,float blend){
	return max(base+blend -1.0,0.0);
}
float BlendLightenf(float base,float blend){
	return max(blend,base);
}
float BlendDarkenf(float base,float blend){
	return min(blend,base);
}
float BlendScreenf(float base,float blend){
	return (1.0 - ((1.0 - (base))*(1.0 - (blend))));
}
float BlendOverlayf(float base,float blend){
	return (base < 0.5 ? (2.0*(base)*(blend)) : (1.0 -2.0*(1.0 - (base))*(1.0 - (blend))));
}
float BlendSoftLightf(float base,float blend){
	return ((1.0 -2.0*(blend))*(base)*(base)+2.0*(base)*(blend));
}
float BlendColorDodgef(float base,float blend){
	return ((blend==1.0) ? blend : min((base)/(1.0 - (blend)),1.0));
}
float BlendColorBurnf(float base,float blend){
	return ((blend==0.0) ? blend : max((1.0 - ((1.0 - (base))/(blend))),0.0));
}
float BlendLinearLightf(float base,float blend){
	if(blend < 0.5){
		return BlendLinearBurnf(base,2.0*blend);
	}
	else {
		return BlendLinearDodgef(base,2.0*(blend -0.5));
	}
}
float BlendVividLightf(float base,float blend){
	if(blend < 0.5){
		return BlendColorBurnf(base,2.0*blend);
	}
	else {
		return BlendColorDodgef(base,2.0*(blend -0.5));
	}
}
float BlendPinLightf(float base,float blend){
	if(blend < 0.5){
		return BlendDarkenf(base,2.0*blend);
	}
	else {
		return BlendLightenf(base,2.0*(blend -0.5));
	}
}
float BlendHardMixf(float base,float blend){
	if(BlendVividLightf(base,blend) < 0.5){
		return 0.0;
	}
	else {
		return 1.0;
	}
}
float BlendReflectf(float base,float blend){
	return ((blend==1.0) ? blend : min((base)*(base)/(1.0 - (blend)),1.0));
}

#define BlendNormal(base,blend) 		(blend)
#define BlendLighten(base,blend)		(vec3(BlendLightenf(base.r,blend.r),BlendLightenf(base.g,blend.g),BlendLightenf(base.b,blend.b)))
#define BlendDarken(base,blend)		(vec3(BlendDarkenf(base.r,blend.r),BlendDarkenf(base.g,blend.g),BlendDarkenf(base.b,blend.b)))
#define BlendMultiply(base,blend) 		((base)*(blend))
#define BlendDivide(base,blend) 		((blend)/(base))
#define BlendAverage(base,blend) 		((base+blend)/2.0)
#define BlendAdd(base,blend) 			min(base+blend,vec3(1.0))
#define BlendSubtract(base,blend) 		max(base+blend -vec3(1.0),vec3(0.0))
#define BlendDifference(base,blend) 	abs(base - (blend))
#define BlendNegation(base,blend) 		(vec3(1.0) -abs(vec3(1.0) - (base) - (blend)))
#define BlendExclusion(base,blend) 	(base+blend -2.0*(base)*(blend))
#define BlendScreen(base,blend) 		vec3(BlendScreenf(base.r,blend.r),BlendScreenf(base.g,blend.g),BlendScreenf(base.b,blend.b))

#define BlendOverlay(base,blend) 		vec3(BlendOverlayf(base.r,blend.r),BlendOverlayf(base.g,blend.g),BlendOverlayf(base.b,blend.b))
#define BlendSoftLight(base,blend) 	vec3(BlendSoftLightf(base.r,blend.r),BlendSoftLightf(base.g,blend.g),BlendSoftLightf(base.b,blend.b))
#define BlendHardLight(base,blend) 	BlendOverlay(blend,base)
#define BlendColorDodge(base,blend) 	vec3(BlendColorDodgef(base.r,blend.r),BlendColorDodgef(base.g,blend.g),BlendColorDodgef(base.b,blend.b))
#define BlendColorBurn(base,blend) 	vec3(BlendColorBurnf(base.r,blend.r),BlendColorBurnf(base.g,blend.g),BlendColorBurnf(base.b,blend.b))
#define BlendLinearDodge(base,blend)	BlendAdd(base,blend)
#define BlendLinearBurn(base,blend)	BlendSubtract(base,blend)

#define BlendLinearLight(base,blend) 	vec3(BlendLinearLightf(base.r,blend.r),BlendLinearLightf(base.g,blend.g),BlendLinearLightf(base.b,blend.b))
#define BlendVividLight(base,blend) 	vec3(BlendVividLightf(base.r,blend.r),BlendVividLightf(base.g,blend.g),BlendVividLightf(base.b,blend.b))
#define BlendPinLight(base,blend) 		vec3(BlendPinLightf(base.r,blend.r),BlendPinLightf(base.g,blend.g),BlendPinLightf(base.b,blend.b))
#define BlendHardMix(base,blend) 		vec3(BlendHardMixf(base.r,blend.r),BlendHardMixf(base.g,blend.g),BlendHardMixf(base.b,blend.b))
#define BlendReflect(base,blend) 		vec3(BlendReflectf(base.r,blend.r),BlendReflectf(base.g,blend.g),BlendReflectf(base.b,blend.b))
#define BlendGlow(base,blend) 			BlendReflect(blend,base)
#define BlendPhoenix(base,blend) 		(min(base,blend) -max(base,blend)+vec3(1.0))
#define BlendOpacity(base,blend,F,O) 	(F(base,blend)*O+(blend)*(1.0 -O))

vec3 BlendHue(vec3 base,vec3 blend)
{
	vec3 baseHSL=RGBToHSL(base);
	return HSLToRGB(vec3(RGBToHSL(blend).r,baseHSL.g,baseHSL.b));
}

vec3 BlendSaturation(vec3 base,vec3 blend)
{
	vec3 baseHSL=RGBToHSL(base);
	return HSLToRGB(vec3(baseHSL.r,RGBToHSL(blend).g,baseHSL.b));
}

vec3 BlendColor(vec3 base,vec3 blend)
{
	vec3 blendHSL=RGBToHSL(blend);
	return HSLToRGB(vec3(blendHSL.r,blendHSL.g,RGBToHSL(base).b));
}

vec3 BlendLuminosity(vec3 base,vec3 blend)
{
	vec3 baseHSL=RGBToHSL(base);
	return HSLToRGB(vec3(baseHSL.r,baseHSL.g,RGBToHSL(blend).b));
}

#define GammaCorrection(color,gamma)								pow(color,1.0/gamma)

#define LevelsControlInputRange(color,minInput,maxInput)				min(max(color -vec3(minInput),vec3(0.0))/(vec3(maxInput) -vec3(minInput)),vec3(1.0))
#define LevelsControlInput(color,minInput,gamma,maxInput)				GammaCorrection(LevelsControlInputRange(color,minInput,maxInput),gamma)
#define LevelsControlOutputRange(color,minOutput,maxOutput) 			mix(vec3(minOutput),vec3(maxOutput),color)
#define LevelsControl(color,minInput,gamma,maxInput,minOutput,maxOutput) 	LevelsControlOutputRange(LevelsControlInput(color,minInput,gamma,maxInput),minOutput,maxOutput)

#if defined BLEND_MODE_NORMAL
#define definedBlend(a,b) BlendNormal(a,b)

#elif defined BLEND_MODE_LIGHTEN
#define definedBlend(a,b) BlendLighten(a,b)

#elif defined BLEND_MODE_DARKEN
#define definedBlend(a,b) BlendDarken(a,b)

#elif defined BLEND_MODE_MULTIPLY
#define definedBlend(a,b) BlendMultiply(a,b)

#elif defined BLEND_MODE_DIVIDE
#define definedBlend(a,b) BlendDivide(a,b)

#elif defined BLEND_MODE_AVERAGE
#define definedBlend(a,b) BlendAverage(a,b)

#elif defined BLEND_MODE_ADD
#define definedBlend(a,b) BlendAdd(a,b)

#elif defined BLEND_MODE_SUBTRACT
#define definedBlend(a,b) BlendSubtract(a,b)

#elif defined BLEND_MODE_DIFFERENCE
#define definedBlend(a,b) BlendDifference(a,b)

#elif defined BLEND_MODE_NEGATION
#define definedBlend(a,b) BlendNegation(a,b)

#elif defined BLEND_MODE_EXCLUSION
#define definedBlend(a,b) BlendExclusion(a,b)

#elif defined BLEND_MODE_SCREEN
#define definedBlend(a,b) BlendScreen(a,b)

#elif defined BLEND_MODE_OVERLAY
#define definedBlend(a,b) BlendOverlay(a,b)

#elif defined BLEND_MODE_SOFT_LIGHT
#define definedBlend(a,b) BlendSoftLight(a,b)

#elif defined BLEND_MODE_HARD_LIGHT
#define definedBlend(a,b) BlendHardLight(a,b)

#elif defined BLEND_MODE_COLOR_DODGE
#define definedBlend(a,b) BlendColorDodge(a,b)

#elif defined BLEND_MODE_COLOR_BURN
#define definedBlend(a,b) BlendColorBurn(a,b)

#elif defined BLEND_MODE_LINEAR_LIGHT
#define definedBlend(a,b) BlendLinearLight(a,b)

#elif defined BLEND_MODE_VIVID_LIGHT
#define definedBlend(a,b) BlendVividLight(a,b)

#elif defined BLEND_MODE_PIN_LIGHT
#define definedBlend(a,b) BlendPinLight(a,b)

#elif defined BLEND_MODE_HARD_MIX
#define definedBlend(a,b) BlendHardMix(a,b)

#elif defined BLEND_MODE_HARD_REFLECT
#define definedBlend(a,b) BlendReflect(a,b)

#elif defined BLEND_MODE_HARD_GLOW
#define definedBlend(a,b) BlendGlow(a,b)

#elif defined BLEND_MODE_HARD_PHOENIX
#define definedBlend(a,b) BlendPhoenix(a,b)

#elif defined BLEND_MODE_HUE
#define definedBlend(a,b) BlendHue(a,b)

#elif defined BLEND_MODE_SATURATION
#define definedBlend(a,b) BlendSaturation(a,b)

#elif defined BLEND_MODE_COLOR
#define definedBlend(a,b) BlendColor(a,b)

#elif defined BLEND_MODE_LUMINOSITY
#define definedBlend(a,b) BlendLuminosity(a,b)

#endif

#ifndef definedBlend
#error If you define sc_BlendMode_Custom,you must also define a BLEND_MODE_*!
#endif

vec4 applyCustomBlend(vec4 color){
	vec4 result;
	vec3 framebuffer=getFramebufferColor().rgb;
	result.rgb=definedBlend(framebuffer,color.rgb);
	result.rgb=mix(framebuffer,result.rgb,color.a);
	result.a=1.0;
	return result;
}

#endif
#endif

#endif

#if !defined(MOBILE) && !NODEFLEX
#define STUDIO
#endif

#if defined(STUDIO)
#define ssConstOrUniformPrecision	uniform NF_PRECISION
#define ssConstOrUniform			uniform
#else
#define ssConstOrUniformPrecision const
#define ssConstOrUniform			const
#endif

#if defined(STUDIO)
#define ssPortConstant(xValue,xUniform)	xUniform
#else
#define ssPortConstant(xValue,xUniform)	xValue
#endif

#define float2 vec2
#define float3 vec3
#define float4 vec4
#define bool2	bvec2
#define bool3	bvec3
#define bool4	bvec4
#define float2x2 mat2
#define float3x3 mat3
#define float4x4 mat4

mat4 ssTranspose(mat4 Matrix)
{
	vec4 i0=Matrix[0];
	vec4 i1=Matrix[1];
	vec4 i2=Matrix[2];
	vec4 i3=Matrix[3];

	return mat4(vec4(i0.x,i1.x,i2.x,i3.x),vec4(i0.y,i1.y,i2.y,i3.y),vec4(i0.z,i1.z,i2.z,i3.z),vec4(i0.w,i1.w,i2.w,i3.w));
}

mat3 ssTranspose(mat3 Matrix)
{
	vec3 i0=Matrix[0];
	vec3 i1=Matrix[1];
	vec3 i2=Matrix[2];

	return mat3(vec3(i0.x,i1.x,i2.x),vec3(i0.y,i1.y,i2.y),vec3(i0.z,i1.z,i2.z));
}

mat2 ssTranspose(mat2 Matrix)
{
	vec2 i0=Matrix[0];
	vec2 i1=Matrix[1];

	return mat2(vec2(i0.x,i1.x),vec2(i0.y,i1.y));
}

float ssConditional(float C,float A,float B){return (bool(C*1.0)) ? A : B;}
float2 ssConditional(float2 C,float A,float2 B){return float2(ssConditional(C.x,A,B.x),ssConditional(C.y,A,B.y));}
float3 ssConditional(float3 C,float A,float3 B){return float3(ssConditional(C.x,A,B.x),ssConditional(C.y,A,B.y),ssConditional(C.z,A,B.z));}
float4 ssConditional(float4 C,float A,float4 B){return float4(ssConditional(C.x,A,B.x),ssConditional(C.y,A,B.y),ssConditional(C.z,A,B.z),ssConditional(C.w,A,B.w));}
float2 ssConditional(float2 C,float A,float B){return float2(ssConditional(C.x,A,B),ssConditional(C.y,A,B));}
float3 ssConditional(float3 C,float A,float B){return float3(ssConditional(C.x,A,B),ssConditional(C.y,A,B),ssConditional(C.z,A,B));}
float4 ssConditional(float4 C,float A,float B){return float4(ssConditional(C.x,A,B),ssConditional(C.y,A,B),ssConditional(C.z,A,B),ssConditional(C.w,A,B));}
float2 ssConditional(float C,float A,float2 B){return float2(ssConditional(C,A,B.x),ssConditional(C,A,B.y));}
float3 ssConditional(float C,float A,float3 B){return float3(ssConditional(C,A,B.x),ssConditional(C,A,B.y),ssConditional(C,A,B.z));}
float4 ssConditional(float C,float A,float4 B){return float4(ssConditional(C,A,B.x),ssConditional(C,A,B.y),ssConditional(C,A,B.z),ssConditional(C,A,B.w));}

float ssEqual(float A,float B){return (A==B) ? 1.0 : 0.0;}
float2 ssEqual(float2 A,float2 B){return float2(ssEqual(A.x,B.x),ssEqual(A.y,B.y));}
float3 ssEqual(float3 A,float3 B){return float3(ssEqual(A.x,B.x),ssEqual(A.y,B.y),ssEqual(A.z,B.z));}
float4 ssEqual(float4 A,float4 B){return float4(ssEqual(A.x,B.x),ssEqual(A.y,B.y),ssEqual(A.z,B.z),ssEqual(A.w,B.w));}
float2 ssEqual(float2 A,float B){return float2(ssEqual(A.x,B),ssEqual(A.y,B));}
float3 ssEqual(float3 A,float B){return float3(ssEqual(A.x,B),ssEqual(A.y,B),ssEqual(A.z,B));}
float4 ssEqual(float4 A,float B){return float4(ssEqual(A.x,B),ssEqual(A.y,B),ssEqual(A.z,B),ssEqual(A.w,B));}
float2 ssEqual(float A,float2 B){return float2(ssEqual(A,B.x),ssEqual(A,B.y));}
float3 ssEqual(float A,float3 B){return float3(ssEqual(A,B.x),ssEqual(A,B.y),ssEqual(A,B.z));}
float4 ssEqual(float A,float4 B){return float4(ssEqual(A,B.x),ssEqual(A,B.y),ssEqual(A,B.z),ssEqual(A,B.w));}

float ssNotEqual(float A,float B){return (A==B) ? 0.0 : 1.0;}
float2 ssNotEqual(float2 A,float2 B){return float2(ssNotEqual(A.x,B.x),ssNotEqual(A.y,B.y));}
float3 ssNotEqual(float3 A,float3 B){return float3(ssNotEqual(A.x,B.x),ssNotEqual(A.y,B.y),ssNotEqual(A.z,B.z));}
float4 ssNotEqual(float4 A,float4 B){return float4(ssNotEqual(A.x,B.x),ssNotEqual(A.y,B.y),ssNotEqual(A.z,B.z),ssNotEqual(A.w,B.w));}
float2 ssNotEqual(float2 A,float B){return float2(ssNotEqual(A.x,B),ssNotEqual(A.y,B));}
float3 ssNotEqual(float3 A,float B){return float3(ssNotEqual(A.x,B),ssNotEqual(A.y,B),ssNotEqual(A.z,B));}
float4 ssNotEqual(float4 A,float B){return float4(ssNotEqual(A.x,B),ssNotEqual(A.y,B),ssNotEqual(A.z,B),ssNotEqual(A.w,B));}
float2 ssNotEqual(float A,float2 B){return float2(ssNotEqual(A,B.x),ssNotEqual(A,B.y));}
float3 ssNotEqual(float A,float3 B){return float3(ssNotEqual(A,B.x),ssNotEqual(A,B.y),ssNotEqual(A,B.z));}
float4 ssNotEqual(float A,float4 B){return float4(ssNotEqual(A,B.x),ssNotEqual(A,B.y),ssNotEqual(A,B.z),ssNotEqual(A,B.w));}

float ssLarger(float A,float B){return (A > B) ? 1.0 : 0.0;}
float2 ssLarger(float2 A,float2 B){return float2(ssLarger(A.x,B.x),ssLarger(A.y,B.y));}
float3 ssLarger(float3 A,float3 B){return float3(ssLarger(A.x,B.x),ssLarger(A.y,B.y),ssLarger(A.z,B.z));}
float4 ssLarger(float4 A,float4 B){return float4(ssLarger(A.x,B.x),ssLarger(A.y,B.y),ssLarger(A.z,B.z),ssLarger(A.w,B.w));}
float2 ssLarger(float2 A,float B){return float2(ssLarger(A.x,B),ssLarger(A.y,B));}
float3 ssLarger(float3 A,float B){return float3(ssLarger(A.x,B),ssLarger(A.y,B),ssLarger(A.z,B));}
float4 ssLarger(float4 A,float B){return float4(ssLarger(A.x,B),ssLarger(A.y,B),ssLarger(A.z,B),ssLarger(A.w,B));}
float2 ssLarger(float A,float2 B){return float2(ssLarger(A,B.x),ssLarger(A,B.y));}
float3 ssLarger(float A,float3 B){return float3(ssLarger(A,B.x),ssLarger(A,B.y),ssLarger(A,B.z));}
float4 ssLarger(float A,float4 B){return float4(ssLarger(A,B.x),ssLarger(A,B.y),ssLarger(A,B.z),ssLarger(A,B.w));}

float ssLargerOrEqual(float A,float B){return (A >=B) ? 1.0 : 0.0;}
float2 ssLargerOrEqual(float2 A,float2 B){return float2(ssLargerOrEqual(A.x,B.x),ssLargerOrEqual(A.y,B.y));}
float3 ssLargerOrEqual(float3 A,float3 B){return float3(ssLargerOrEqual(A.x,B.x),ssLargerOrEqual(A.y,B.y),ssLargerOrEqual(A.z,B.z));}
float4 ssLargerOrEqual(float4 A,float4 B){return float4(ssLargerOrEqual(A.x,B.x),ssLargerOrEqual(A.y,B.y),ssLargerOrEqual(A.z,B.z),ssLargerOrEqual(A.w,B.w));}
float2 ssLargerOrEqual(float2 A,float B){return float2(ssLargerOrEqual(A.x,B),ssLargerOrEqual(A.y,B));}
float3 ssLargerOrEqual(float3 A,float B){return float3(ssLargerOrEqual(A.x,B),ssLargerOrEqual(A.y,B),ssLargerOrEqual(A.z,B));}
float4 ssLargerOrEqual(float4 A,float B){return float4(ssLargerOrEqual(A.x,B),ssLargerOrEqual(A.y,B),ssLargerOrEqual(A.z,B),ssLargerOrEqual(A.w,B));}
float2 ssLargerOrEqual(float A,float2 B){return float2(ssLargerOrEqual(A,B.x),ssLargerOrEqual(A,B.y));}
float3 ssLargerOrEqual(float A,float3 B){return float3(ssLargerOrEqual(A,B.x),ssLargerOrEqual(A,B.y),ssLargerOrEqual(A,B.z));}
float4 ssLargerOrEqual(float A,float4 B){return float4(ssLargerOrEqual(A,B.x),ssLargerOrEqual(A,B.y),ssLargerOrEqual(A,B.z),ssLargerOrEqual(A,B.w));}

float ssSmaller(float A,float B){return (A < B) ? 1.0 : 0.0;}
float2 ssSmaller(float2 A,float2 B){return float2(ssSmaller(A.x,B.x),ssSmaller(A.y,B.y));}
float3 ssSmaller(float3 A,float3 B){return float3(ssSmaller(A.x,B.x),ssSmaller(A.y,B.y),ssSmaller(A.z,B.z));}
float4 ssSmaller(float4 A,float4 B){return float4(ssSmaller(A.x,B.x),ssSmaller(A.y,B.y),ssSmaller(A.z,B.z),ssSmaller(A.w,B.w));}
float2 ssSmaller(float2 A,float B){return float2(ssSmaller(A.x,B),ssSmaller(A.y,B));}
float3 ssSmaller(float3 A,float B){return float3(ssSmaller(A.x,B),ssSmaller(A.y,B),ssSmaller(A.z,B));}
float4 ssSmaller(float4 A,float B){return float4(ssSmaller(A.x,B),ssSmaller(A.y,B),ssSmaller(A.z,B),ssSmaller(A.w,B));}
float2 ssSmaller(float A,float2 B){return float2(ssSmaller(A,B.x),ssSmaller(A,B.y));}
float3 ssSmaller(float A,float3 B){return float3(ssSmaller(A,B.x),ssSmaller(A,B.y),ssSmaller(A,B.z));}
float4 ssSmaller(float A,float4 B){return float4(ssSmaller(A,B.x),ssSmaller(A,B.y),ssSmaller(A,B.z),ssSmaller(A,B.w));}

float ssSmallerOrEqual(float A,float B){return (A <=B) ? 1.0 : 0.0;}
float2 ssSmallerOrEqual(float2 A,float2 B){return float2(ssSmallerOrEqual(A.x,B.x),ssSmallerOrEqual(A.y,B.y));}
float3 ssSmallerOrEqual(float3 A,float3 B){return float3(ssSmallerOrEqual(A.x,B.x),ssSmallerOrEqual(A.y,B.y),ssSmallerOrEqual(A.z,B.z));}
float4 ssSmallerOrEqual(float4 A,float4 B){return float4(ssSmallerOrEqual(A.x,B.x),ssSmallerOrEqual(A.y,B.y),ssSmallerOrEqual(A.z,B.z),ssSmallerOrEqual(A.w,B.w));}
float2 ssSmallerOrEqual(float2 A,float B){return float2(ssSmallerOrEqual(A.x,B),ssSmallerOrEqual(A.y,B));}
float3 ssSmallerOrEqual(float3 A,float B){return float3(ssSmallerOrEqual(A.x,B),ssSmallerOrEqual(A.y,B),ssSmallerOrEqual(A.z,B));}
float4 ssSmallerOrEqual(float4 A,float B){return float4(ssSmallerOrEqual(A.x,B),ssSmallerOrEqual(A.y,B),ssSmallerOrEqual(A.z,B),ssSmallerOrEqual(A.w,B));}
float2 ssSmallerOrEqual(float A,float2 B){return float2(ssSmallerOrEqual(A,B.x),ssSmallerOrEqual(A,B.y));}
float3 ssSmallerOrEqual(float A,float3 B){return float3(ssSmallerOrEqual(A,B.x),ssSmallerOrEqual(A,B.y),ssSmallerOrEqual(A,B.z));}
float4 ssSmallerOrEqual(float A,float4 B){return float4(ssSmallerOrEqual(A,B.x),ssSmallerOrEqual(A,B.y),ssSmallerOrEqual(A,B.z),ssSmallerOrEqual(A,B.w));}

float ssAll(float A){return (bool(A*1.0)) ? 1.0 : 0.0;}
float ssAll(float2 A){return all(notEqual(A,float2(0.0))) ? 1.0 : 0.0;}
float ssAll(float3 A){return all(notEqual(A,float3(0.0))) ? 1.0 : 0.0;}
float ssAll(float4 A){return all(notEqual(A,float4(0.0))) ? 1.0 : 0.0;}

float ssAny(float A){return (bool(A*1.0)) ? 1.0 : 0.0;}
float ssAny(float2 A){return any(notEqual(A,float2(0.0))) ? 1.0 : 0.0;}
float ssAny(float3 A)	{return any(notEqual(A,float3(0.0))) ? 1.0 : 0.0;}
float ssAny(float4 A)	{return any(notEqual(A,float4(0.0))) ? 1.0 : 0.0;}

float ssNot(float A){return (bool(A*1.0)) ? 0.0 : 1.0;}
float2 ssNot(float2 A){return float2(ssNot(A.x),ssNot(A.y));}
float3 ssNot(float3 A){return float3(ssNot(A.x),ssNot(A.y),ssNot(A.z));}
float4 ssNot(float4 A){return float4(ssNot(A.x),ssNot(A.y),ssNot(A.z),ssNot(A.w));}

float tempSrgbToLinear(float x)
{
	#if defined(DEVICE_IS_FAST)
	return pow(x,2.2);
	#else
	return x*x;
	#endif
}

float ssSRGB_to_Linear(float A){return tempSrgbToLinear(A);}
float2 ssSRGB_to_Linear(float2 A){return float2(tempSrgbToLinear(A.r),tempSrgbToLinear(A.g));}
float3 ssSRGB_to_Linear(float3 A){return float3(tempSrgbToLinear(A.r),tempSrgbToLinear(A.g),tempSrgbToLinear(A.b));}
float4 ssSRGB_to_Linear(float4 A){return float4(tempSrgbToLinear(A.r),tempSrgbToLinear(A.g),tempSrgbToLinear(A.b),A.a);}

float tempLinearToSrgb(float x)
{
	#if defined(DEVICE_IS_FAST)
	return pow(x,1.0/2.2);
	#else
	return sqrt(x);
	#endif
}

float ssLinear_to_SRGB(float A){return tempLinearToSrgb(A);}
float2 ssLinear_to_SRGB(float2 A){return float2(tempLinearToSrgb(A.r),tempLinearToSrgb(A.g));}
float3 ssLinear_to_SRGB(float3 A){return float3(tempLinearToSrgb(A.r),tempLinearToSrgb(A.g),tempLinearToSrgb(A.b));}
float4 ssLinear_to_SRGB(float4 A){return float4(tempLinearToSrgb(A.r),tempLinearToSrgb(A.g),tempLinearToSrgb(A.b),A.a);}

float Dummy1;
float2 Dummy2;
float3 Dummy3;
float4 Dummy4;

#if defined(STUDIO)

struct ssPreviewInfo
{
	float4 Color;
	bool Saved;
};

uniform NF_PRECISION int PreviewEnabled;
uniform NF_PRECISION int PreviewNodeID;

varying float4 PreviewVertexColor;
varying float PreviewVertexSaved;

#define ssDisableVertexChanges 					(PreviewEnabled==1)

#define ssFunctionDefs 							ssGlobals Globals,inout ssPreviewInfo PreviewInfo
#define ssFunctionInputs 						Globals,PreviewInfo

#define ssSetupPreview_Vertex					ssPreviewInfo PreviewInfo;PreviewInfo.Color=PreviewVertexColor=float4(0.5);PreviewInfo.Saved=false;PreviewVertexSaved=0.0;
#define ssSetupPreview_Pixel					ssPreviewInfo PreviewInfo;PreviewInfo.Color=PreviewVertexColor;PreviewInfo.Saved=(bool(PreviewVertexSaved*1.0)) ? true : false;
#define ssPreviewSave(xCode,xNodeID,xAlpha) if (PreviewEnabled==1 && !PreviewInfo.Saved && xNodeID==PreviewNodeID){PreviewInfo.Saved=true;{PreviewInfo.Color=xCode;if (!xAlpha) PreviewInfo.Color.a=1.0;}}
#define ssPreviewForceSave(xCode) 			if (PreviewEnabled==0){PreviewInfo.Saved=true;{PreviewInfo.Color=xCode;}}
#define ssPreviewOutput_Vertex					if (PreviewInfo.Saved){PreviewVertexColor=float4(PreviewInfo.Color.rgb,1.0);PreviewVertexSaved=1.0;}
#define ssPreviewOutput_Pixel					if (PreviewEnabled==1){if (PreviewInfo.Saved){FinalColor=float4(PreviewInfo.Color);} else {FinalColor=vec4(0.0,0.0,0.0,0.0);}}

#else

#define ssDisableVertexChanges				false

#define ssFunctionDefs 						ssGlobals Globals
#define ssFunctionInputs 					Globals

#define ssSetupPreview_Vertex
#define ssSetupPreview_Pixel
#define ssPreviewSave(xCode,xNodeID,xAlpha)
#define ssPreviewForceSave(xCode)
#define ssPreviewOutput_Vertex
#define ssPreviewOutput_Pixel

#endif

#ifdef FRAGMENT_SHADER
#ifdef GL_FRAGMENT_PRECISION_HIGH
#define ssTimeElapsed 	float(sc_TimeElapsed)
#else
#define ssTimeElapsed 	Globals.TimeElapsed
#endif
#else
#define ssTimeElapsed 		float(sc_TimeElapsed)
#endif

#ifdef FRAGMENT_SHADER
#ifdef GL_FRAGMENT_PRECISION_HIGH
#define ssTimeDelta 	float(sc_TimeDelta)
#else
#define ssTimeDelta		Globals.TimeDelta
#endif
#else
#define ssTimeDelta			float(sc_TimeDelta)
#endif

#ifdef VERTEX_SHADER

attribute vec4 color;

void ngsVertexShaderBegin(out sc_Vertex_t v)
{
	v=sc_LoadVertexAttributes();

	sc_BlendVertex(v);
	sc_SkinVertex(v);

	#if (sc_RenderingSpace==SC_RENDERING_SPACE_SCREEN)

	varPos	=vec3(0.0);
	varNormal=v.normal;
	varTangent=v.tangent;

	#elif sc_RenderingSpace==SC_RENDERING_SPACE_SCREEN_MV

	varPos	=vec3(0.0);
	varNormal=v.normal;
	varTangent=v.tangent;

	#elif sc_RenderingSpace==SC_RENDERING_SPACE_WORLD

	varPos	=v.position.xyz;
	varNormal=v.normal;
	varTangent=v.tangent;

	#elif sc_RenderingSpace==SC_RENDERING_SPACE_OBJECT

	varPos	=(sc_ModelMatrix*v.position).xyz;
	varNormal=sc_NormalMatrix*v.normal;
	varTangent=sc_NormalMatrix*v.tangent;

	#else

	#error Unknown sc_RenderingSpace!

	#endif

	varColor=color;
}

void ngsVertexShaderEnd(inout sc_Vertex_t v,vec3 WorldPosition,vec3 WorldNormal,vec3 WorldTangent,vec4 ScreenPosition)
{
	varPos	=WorldPosition;
	varNormal=WorldNormal;
	varTangent=WorldTangent;

	#if (sc_RenderingSpace==SC_RENDERING_SPACE_SCREEN)

	float4 screenPosition=ScreenPosition;

	#elif sc_RenderingSpace==SC_RENDERING_SPACE_SCREEN_MV

	float4 screenPosition=(sc_ModelViewMatrix*v.position)*vec4(1.0/sc_Camera.aspect,1.0,1.0,1.0);

	#elif sc_RenderingSpace==SC_RENDERING_SPACE_WORLD

	float4 screenPosition=sc_ViewProjectionMatrix*float4(varPos.xyz,1.0);

	#elif sc_RenderingSpace==SC_RENDERING_SPACE_OBJECT

	float4 screenPosition=sc_ViewProjectionMatrix*float4(varPos.xyz,1.0);

	#else

	#error Unknown sc_RenderingSpace!

	#endif

	varNormal		=normalize(varNormal);
	varTangent		=normalize(varTangent);
	varBitangentSign	=tangent.w;
	varTex0			=v.texture0;
	varTex1			=v.texture1;
	varScreenPos		=screenPosition;
	varScreenTexturePos=screenPosition.xy;

	#if defined(sc_ProjectiveShadowsReceiver)

	varShadowTex=getProjectedTexCoords(v.position);

	#endif

	#if (sc_DepthBufferMode==SC_DEPTH_BUFFER_MODE_LOGARITHMIC)

	float fCoefficient=2.0/log2(sc_Camera.clipPlanes.y+1.0);
	screenPosition.z=(log2(max(sc_Camera.clipPlanes.x,1.0+screenPosition.w))*fCoefficient -1.0)*screenPosition.w;

	#endif

	gl_Position=screenPosition;
}

#endif

#ifdef FRAGMENT_SHADER

#ifdef RENDER_CONSTANT_COLOR

void main()
{
	gl_FragColor=baseColor;
}

#else

uniform NF_PRECISION float alphaTestThreshold;

void ngsAlphaTest(float opacity)
{
	#if defined(sc_BlendMode_AlphaTest)

	if (opacity < alphaTestThreshold)
	{
		discard;
	}

	#endif

	#if defined(ENABLE_STIPPLE_PATTERN_TEST)

	vec2 localCoord=floor(mod(gl_FragCoord.xy,vec2(4.0)));
	float threshold=(mod(dot(localCoord,vec2(4.0,1.0))*9.0,16.0)+1.0)/17.0;

	if (opacity < threshold)
	{
		discard;
	}

	#endif
}

vec4 ngsPixelShader(vec4 result)
{
	#if defined(sc_ProjectiveShadowsCaster)

	return evaluateShadowCasterColor(result);

	#else

	#if defined(sc_BlendMode_Custom)

	result=applyCustomBlend(result);

	#elif defined(sc_BlendMode_MultiplyOriginal)

	result.rgb=mix(vec3(1.0),result.rgb,result.a);

	#elif defined(sc_BlendMode_Screen)

	result.rgb=result.rgb*result.a;

	#endif

	return result;

	#endif
}

#endif

#endif

#ifdef VERTEX_SHADER

#define SC_DISABLE_FRUSTUM_CULLING

varying vec4 Interpolator3;

uniform NF_PRECISION		mat4 realModelMatrix;
uniform sampler2D baseTex;
uniform mat3 baseTexTransform;
uniform vec4 baseTexUvMinMax;
uniform vec4 baseTexBorderColor;
uniform vec4 baseTexSize;

#ifndef SC_USE_UV_TRANSFORM_baseTex
#define SC_USE_UV_TRANSFORM_baseTex false
#define SC_SOFTWARE_WRAP_MODE_U_baseTex SOFTWARE_WRAP_MODE_DISABLED
#define SC_SOFTWARE_WRAP_MODE_V_baseTex SOFTWARE_WRAP_MODE_DISABLED
#define SC_USE_UV_MIN_MAX_baseTex false
#endif
#ifndef SC_USE_CLAMP_TO_BORDER_baseTex
#define SC_USE_CLAMP_TO_BORDER_baseTex false
#endif

#ifndef baseTexUV
#define baseTexUV 0
#endif
uniform NF_PRECISION		mat4 uv2Matrix;

#if defined(STUDIO)
uniform NF_PRECISION float Port_RangeMinA_N003;
uniform NF_PRECISION float Port_RangeMaxA_N003;
uniform NF_PRECISION float Port_RangeMinB_N003;
uniform NF_PRECISION float Port_RangeMaxB_N003;
uniform NF_PRECISION float Port_Value2_N019;
uniform NF_PRECISION float Port_Value2_N005;
uniform NF_PRECISION float Port_RangeMinA_N011;
uniform NF_PRECISION float Port_RangeMaxA_N011;
uniform NF_PRECISION float Port_RangeMinB_N011;
uniform NF_PRECISION float Port_RangeMaxB_N011;
#endif

struct ssGlobals
{
	bool InVertexShader;
	bool InPixelShader;
	float2 Surface_UVCoord0;
	float3 VertexNormal_WorldSpace;
	float3 VertexNormal_ObjectSpace;
	float3 SurfacePosition_ObjectSpace;
};

void Node0_Surface_UV_Coord_0(out float2 UVCoord,ssFunctionDefs)
{
	UVCoord=Globals.Surface_UVCoord0;
}

void Node3_Remap(in float2 ValueIn,out float2 ValueOut,ssFunctionDefs)
{
	float RangeMinA=ssPortConstant(float(0.0),Port_RangeMinA_N003);
	float RangeMaxA=ssPortConstant(float(1.0),Port_RangeMaxA_N003);
	float RangeMinB=ssPortConstant(float(-1.0),Port_RangeMinB_N003);
	float RangeMaxB=ssPortConstant(float(1.0),Port_RangeMaxB_N003);

	ValueOut=((ValueIn -RangeMinA)/(RangeMaxA -RangeMinA))*(RangeMaxB -RangeMinB)+RangeMinB;
}

void Node17_Transform_Vector(in float4 VectorIn,out float3 VectorOut,ssFunctionDefs)
{

	float4 Temp=sc_ViewProjectionMatrixInverse*float4(VectorIn.xyz,1.0);
	VectorOut.xyz=Temp.xyz/Temp.w;
}

void Node12_Matrix_Parameter(out mat4 Output,ssFunctionDefs)
{
	Output=realModelMatrix;
}

void Node15_Surface_Normal(out float3 Normal,ssFunctionDefs)
{
	Normal=Globals.VertexNormal_ObjectSpace;
}

void Node19_Construct_Vector(in float3 Value1,out float4 Value,ssFunctionDefs)
{
	float Value2=ssPortConstant(float(0.0),Port_Value2_N019);

	Value.xyz=Value1;
	Value.w=Value2;
}

void Node16_Transform_by_Matrix(in mat4 Input0,in float4 Input1,out float4 Output,ssFunctionDefs)
{
	Output=Input0*Input1;
}

void Node20_Normalize(in float4 Input0,out float4 Output,ssFunctionDefs)
{
	Output=normalize(Input0);
}

void Node21_Transform_Vector(in float4 VectorIn,out float3 VectorOut,ssFunctionDefs)
{

	float4 Temp=sc_ViewProjectionMatrixInverse*float4(VectorIn.xyz,1.0);
	VectorOut.xyz=Temp.xyz/Temp.w;
}

void Node13_Matrix_Parameter(out mat4 Output,ssFunctionDefs)
{
	Output=uv2Matrix;
}

void Node7_Surface_Position(out float3 Position,ssFunctionDefs)
{
	Position=Globals.SurfacePosition_ObjectSpace;
}

void Node5_Construct_Vector(in float3 Value1,out float4 Value,ssFunctionDefs)
{
	float Value2=ssPortConstant(float(1.0),Port_Value2_N005);

	Value.xyz=Value1;
	Value.w=Value2;
}

void Node14_Transform_by_Matrix(in mat4 Input0,in float4 Input1,out float4 Output,ssFunctionDefs)
{
	Output=Input0*Input1;
}

void Node6_Transform_by_Matrix(in mat4 Input0,in float4 Input1,out float4 Output,ssFunctionDefs)
{
	Output=Input0*Input1;
}

void Node8_Swizzle(in float2 Input,out float2 Output,ssFunctionDefs)
{
	Output=float2(Input.x,Input.y);
}

void Node9_Swizzle(in float4 Input,out float Output,ssFunctionDefs)
{
	Output=Input.w;
}

void Node10_Divide(in float2 Input0,in float Input1,out float2 Output,ssFunctionDefs)
{
	Output=Input0/float2(Input1);
}

void Node11_Remap(in float2 ValueIn,out float2 ValueOut,ssFunctionDefs)
{
	float RangeMinA=ssPortConstant(float(-1.0),Port_RangeMinA_N011);
	float RangeMaxA=ssPortConstant(float(1.0),Port_RangeMaxA_N011);
	float RangeMinB=ssPortConstant(float(0.0),Port_RangeMinB_N011);
	float RangeMaxB=ssPortConstant(float(1.0),Port_RangeMaxB_N011);

	ValueOut=((ValueIn -RangeMinA)/(RangeMaxA -RangeMinA))*(RangeMaxB -RangeMinB)+RangeMinB;
}

void Interpolate_N18(ssFunctionDefs)
{
	float2 Input0=float2(0.0,0.0);;

	{
		mat4 Output_N13;
		Node13_Matrix_Parameter(Output_N13,ssFunctionInputs);

		mat4 Output_N12;
		Node12_Matrix_Parameter(Output_N12,ssFunctionInputs);

		float3 Position_N7;
		Node7_Surface_Position(Position_N7,ssFunctionInputs);

		float4 Value_N5;
		Node5_Construct_Vector(Position_N7,Value_N5,ssFunctionInputs);

		float4 Output_N14;
		Node14_Transform_by_Matrix(Output_N12,Value_N5,Output_N14,ssFunctionInputs);

		float4 Output_N6;
		Node6_Transform_by_Matrix(Output_N13,Output_N14,Output_N6,ssFunctionInputs);

		float2 Output_N8;
		Node8_Swizzle(Output_N6.xy,Output_N8,ssFunctionInputs);

		float Output_N9;
		Node9_Swizzle(Output_N6,Output_N9,ssFunctionInputs);

		float2 Output_N10;
		Node10_Divide(Output_N8,Output_N9,Output_N10,ssFunctionInputs);

		float2 ValueOut_N11;
		Node11_Remap(Output_N10,ValueOut_N11,ssFunctionInputs);

		Input0=ValueOut_N11;
	}
	Interpolator3.x=Input0.x;
	Interpolator3.y=Input0.y;

}

void main()
{
	ssSetupPreview_Vertex

	sc_Vertex_t v;
	ngsVertexShaderBegin(v);

	ssGlobals Globals;
	Globals.InVertexShader			=true;
	Globals.InPixelShader			=false;
	Globals.Surface_UVCoord0			=v.texture0;
	Globals.VertexNormal_WorldSpace	=varNormal;
	Globals.VertexNormal_ObjectSpace	=(sc_ModelMatrixInverse*float4(Globals.VertexNormal_WorldSpace,0.0)).xyz;
	Globals.SurfacePosition_ObjectSpace=(sc_ModelMatrixInverse*float4(varPos,1.0)).xyz;

	Interpolate_N18(ssFunctionInputs);

	float4 ScreenPosition=vec4(0.0);
	float3 WorldPosition=varPos;
	float3 WorldNormal	=varNormal;
	float3 WorldTangent=varTangent;

	{
		float2 UVCoord_N0;
		Node0_Surface_UV_Coord_0(UVCoord_N0,ssFunctionInputs);

		float2 ValueOut_N3;
		Node3_Remap(UVCoord_N0,ValueOut_N3,ssFunctionInputs);

		float3 VectorOut_N17;
		Node17_Transform_Vector(float4(ValueOut_N3.xy,0.0,0.0),VectorOut_N17,ssFunctionInputs);

		WorldPosition=VectorOut_N17;
	}

	{
		mat4 Output_N12;
		Node12_Matrix_Parameter(Output_N12,ssFunctionInputs);

		float3 Normal_N15;
		Node15_Surface_Normal(Normal_N15,ssFunctionInputs);

		float4 Value_N19;
		Node19_Construct_Vector(Normal_N15,Value_N19,ssFunctionInputs);

		float4 Output_N16;
		Node16_Transform_by_Matrix(Output_N12,Value_N19,Output_N16,ssFunctionInputs);

		float4 Output_N20;
		Node20_Normalize(Output_N16,Output_N20,ssFunctionInputs);

		float3 VectorOut_N21;
		Node21_Transform_Vector(Output_N20,VectorOut_N21,ssFunctionInputs);

		WorldNormal=VectorOut_N21;
	}

	if (ssDisableVertexChanges)
	{
		WorldPosition=varPos;
		WorldNormal	=varNormal;
		WorldTangent=varTangent;
	}

	ngsVertexShaderEnd(v,WorldPosition,WorldNormal,WorldTangent,v.position);

	ssPreviewOutput_Vertex
}

#endif

#ifdef FRAGMENT_SHADER

#define SC_DISABLE_FRUSTUM_CULLING

varying vec4 Interpolator3;

uniform NF_PRECISION		mat4 realModelMatrix;
uniform sampler2D baseTex;
uniform mat3 baseTexTransform;
uniform vec4 baseTexUvMinMax;
uniform vec4 baseTexBorderColor;
uniform vec4 baseTexSize;

#ifndef SC_USE_UV_TRANSFORM_baseTex
#define SC_USE_UV_TRANSFORM_baseTex false
#define SC_SOFTWARE_WRAP_MODE_U_baseTex SOFTWARE_WRAP_MODE_DISABLED
#define SC_SOFTWARE_WRAP_MODE_V_baseTex SOFTWARE_WRAP_MODE_DISABLED
#define SC_USE_UV_MIN_MAX_baseTex false
#endif
#ifndef SC_USE_CLAMP_TO_BORDER_baseTex
#define SC_USE_CLAMP_TO_BORDER_baseTex false
#endif

#ifndef baseTexUV
#define baseTexUV 0
#endif
uniform NF_PRECISION		mat4 uv2Matrix;

#if defined(STUDIO)
uniform NF_PRECISION float Port_Value2_N001;
#endif

struct ssGlobals
{
	bool InVertexShader;
	bool InPixelShader;
};

void Node18_Interpolate(out float2 Output,ssFunctionDefs)
{
	float2 Input0=float2(0.0,0.0);

	Output.x=Interpolator3.x;
	Output.y=Interpolator3.y;
}

void Node2_Texture_2D_Parameter(in float2 UVCoord,out float4 Output,ssFunctionDefs)
{
		vec4 baseTexSample;
	SAMPLE_TEXTURE(baseTexSample,baseTex,UVCoord,SC_USE_UV_TRANSFORM_baseTex,baseTexTransform,SC_SOFTWARE_WRAP_MODE_U_baseTex,SC_SOFTWARE_WRAP_MODE_V_baseTex,SC_USE_UV_MIN_MAX_baseTex,baseTexUvMinMax,SC_USE_CLAMP_TO_BORDER_baseTex,baseTexBorderColor,0.0);;
	Output=baseTexSample;
}

void Node1_Construct_Vector(in float3 Value1,out float4 Value,ssFunctionDefs)
{
	float Value2=ssPortConstant(float(1.0),Port_Value2_N001);

	Value.xyz=Value1;
	Value.w=Value2;
}

#if !defined(RENDER_CONSTANT_COLOR)

void main()
{
	ssSetupPreview_Pixel

	float4 FinalColor=float4(1.0,1.0,1.0,1.0);

	ssGlobals Globals;
	Globals.InVertexShader=false;
	Globals.InPixelShader=true;

	{
		float2 Output_N18;
		Node18_Interpolate(Output_N18,ssFunctionInputs);

		float4 Output_N2;
		Node2_Texture_2D_Parameter(Output_N18,Output_N2,ssFunctionInputs);

		float4 Value_N1;
		Node1_Construct_Vector(Output_N2.xyz,Value_N1,ssFunctionInputs);

		FinalColor=Value_N1;
	}
	ngsAlphaTest(FinalColor.a);

	FinalColor=ngsPixelShader(FinalColor);

	ssPreviewOutput_Pixel

	sc_FragData0=max(FinalColor,0.0);

	#if (defined(sc_OITCompositing4Layer) || defined(sc_OITCompositing8Layer) || defined(sc_OITDepthGather))
	processOIT(sc_FragData0);
	#endif

	#ifdef STUDIO
	vec4 Cost=getPixelRenderingCost();
	if (Cost.w > 0.0)
	sc_FragData0=Cost;
	#endif
}

#endif

#endif

#if sc_IsEditor
#error This is an exported shader. Please do not use shaders in Studio that have already been exported to a lens! Only use fresh shaders,presets,or shaders from existing Studio projects!
#endif

/// Exported with Lens Studio 3.3.0.0
